from __future__ import annotations

from typing import TYPE_CHECKING

from tensorflow import keras

from trans_finder.flares.helpers_debugging import get_debug_output_filepath
from trans_finder.flares.model_utils.utils import get_batch_size
from trans_finder.flares.plot_list.autoencoder import (
    autoenc_create_all_plots_from_model_outputs,
)

if TYPE_CHECKING:
    from trans_finder.flares.manager import Manager


class CreateModelOutputPlotsCallback(keras.callbacks.Callback):
    """Creates plots of the hidden representations at the end of every epoch"""

    def __init__(self, mngr, dataset_train, dataset_eval):
        self.dataset_train = dataset_train
        self.dataset_eval = dataset_eval
        self.mngr: Manager = mngr

        self.is_first_epoch = True

    def create_plots(self, data_desc):
        model_outputs = self.model.predict(
            self.dataset_train, get_batch_size(self, force_predict_size=True)
        )

        if self.mngr.train_phase == 1:
            out_file_name = get_debug_output_filepath(
                self.mngr, "model_plots", f"{data_desc}_hidden_repr.pdf"
            )

            autoenc_create_all_plots_from_model_outputs(
                self.mngr, model_outputs, out_file_name
            )

    def on_epoch_end(self, epoch, logs=None):
        self.create_plots(f"e{epoch}_train")
        pass

    def on_epoch_begin(self, epoch, logs=None):
        if self.is_first_epoch:
            data_desc = "e_initial" if epoch == 0 else f"e{epoch - 1}_train"
            self.create_plots(data_desc)
            self.is_first_epoch = False
