from __future__ import annotations

from typing import TYPE_CHECKING

from tensorflow import keras

from trans_finder.flares.helpers_debugging import (
    get_debug_output_filepath,
    get_tfdataset_input_as_numpy_array,
)
from trans_finder.flares.plot_list.data_normalisation import (
    make_superimposed_feature_dist_timesteps_plot,
)

if TYPE_CHECKING:
    from trans_finder.flares.manager import Manager


class InspectDataNormalisationCallback(keras.callbacks.Callback):
    def __init__(self, mngr, dataset_train, dataset_val):
        super().__init__()
        self.dataset_train = dataset_train
        self.dataset_val = dataset_val
        self.mngr: Manager = mngr

    def on_train_begin(self, logs=None):
        d1_train = get_tfdataset_input_as_numpy_array(self.dataset_train, "dec")

        file_name = get_debug_output_filepath(
            self.mngr,
            "inspect_data_normalisation",
            "training_train_split_d1_kde.pdf",
        )
        make_superimposed_feature_dist_timesteps_plot(self.mngr, d1_train, file_name)

        d1_val = get_tfdataset_input_as_numpy_array(self.dataset_val, "dec")
        file_name = get_debug_output_filepath(
            self.mngr,
            "inspect_data_normalisation",
            "training_eval_split_d1_kde.pdf",
        )
        make_superimposed_feature_dist_timesteps_plot(self.mngr, d1_val, file_name)

        # raise Exception(
        #     "The debug flag debug_inspect_data_normalisation is active. Stopping the execution here."
        # )
