from typing import List

import numpy as np


def create_channel_group_mask(mngr, channel_group: List[str]):
    mask = np.zeros((mngr.n_steps_dec, mngr.n_main_cols))
    keep_indices = [
        (step_channel in channel_group)
        for (step_channel, _) in mngr.data_step_col_names
    ]
    mask[:, keep_indices] = 1

    return mask


def masked_channel_groups_inputs(mngr, predict_in):
    """Modify weights of predict_in by masking channels as specified in the config.
    This is an iterator over all versions."""

    for group_name, group_channels in mngr.channel_groups.items():
        mask = create_channel_group_mask(mngr, group_channels)

        # modify weights by multiplying the channel mask
        modified_input = dict(predict_in.items())
        orig_wgts = predict_in[mngr.tag_wgt]
        new_wgts = np.multiply(orig_wgts, mask)
        modified_input[mngr.tag_wgt] = new_wgts

        yield group_name, modified_input
