from trans_finder.flares.checks.generic_checks import plot_dim_hists_of_ndarrays
from trans_finder.flares.fit_mixture_model import evaluate_mixture_model_fit
from trans_finder.flares.helpers_debugging import get_debug_output_filepath


def compare_inputs_to_calib_and_bgmmfit(
    mngr, model_outputs, bgmmodel, normed_observations
):
    if mngr.run_phase == "calib":
        print("For debugging, comparing calib and bgmm_fit inputs...")
        if not hasattr(mngr, "calib_plot_batch"):
            mngr.calib_plot_batch = 0
        else:
            mngr.calib_plot_batch += 1

        bgmm_fit_outputs_train = temp_load("bgmm_fit_outputs_train")
        bgmm_fit_outputs_val = temp_load("bgmm_fit_outputs_val")

        for key in bgmm_fit_outputs_train.keys():
            plot_dim_hists_of_ndarrays(
                mngr,
                [
                    bgmm_fit_outputs_train[key],
                    bgmm_fit_outputs_val[key],
                    model_outputs[key],
                ],
                ["bgmm_fit_train", "bgmm_fit_val", "calib"],
                get_debug_output_filepath(mngr, "bgmmodel", f"temp_{key}.pdf"),
                # subplots_kwargs=dict(sharex=True),
                hist_kwargs=dict(density=True),
            )

        plot_dim_hists_of_ndarrays(
            mngr,
            [
                bgmm_fit_outputs_train["model_auto_enc_input"],
                bgmm_fit_outputs_train["model_auto_enc_reconstruction"],
            ],
            ["model_auto_enc_input", "model_auto_enc_reconstruction"],
            get_debug_output_filepath(mngr, "bgmmodel", "temp_bgmm_train_ae.pdf"),
            # subplots_kwargs=dict(sharex=True),
            hist_kwargs=dict(density=True),
        )

        plot_dim_hists_of_ndarrays(
            mngr,
            [
                model_outputs["model_auto_enc_input"],
                model_outputs["model_auto_enc_reconstruction"],
            ],
            ["model_auto_enc_input", "model_auto_enc_reconstruction"],
            get_debug_output_filepath(mngr, "bgmmodel", "temp_calib_ae.pdf"),
            # subplots_kwargs=dict(sharex=True),
            hist_kwargs=dict(density=True),
        )

        evaluate_mixture_model_fit(
            mngr,
            bgmmodel,
            normed_observations,
            mngr.mcmc_states[mngr.tag_bgmm_quality_of_fit],
            None,
            data_desc=f"calib_data_{mngr.calib_plot_batch}",
        )
