import os

import numpy as np
from matplotlib import pyplot as plt

from trans_finder.flares.helpers_debugging import get_debug_output_filepath
from trans_finder.flares.plot_list.util.plot_config import gridlines_config, tick_params
from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def check_spread_of_input_error_sample_ts(mngr, df_error_samples_grouped_ts):
    # --- this function is called several times on batches of inputs. If we can change that, the block below can go away.
    if not hasattr(mngr, "check_spread_of_input_error_sample_ts_i_batch"):
        mngr.check_spread_of_input_error_sample_ts_i_batch = 0
    else:
        mngr.check_spread_of_input_error_sample_ts_i_batch += 1

    # --- block end ---

    ts_relative_std = (
        df_error_samples_grouped_ts.std() / df_error_samples_grouped_ts.mean()
    )

    fig, ax = plt.subplots(figsize=(10, 6))
    ax.grid(**gridlines_config)
    ax.tick_params(**tick_params)

    bins = np.linspace(
        ts_relative_std.min().min(),
        ts_relative_std.max().max(),
        30,
    )

    ax.hist(
        ts_relative_std.to_numpy(),
        bins,
        density=False,
    )
    ax.set_xlabel("test statistic std/mean from all input-error-samples per event")
    ax.set_ylabel("Count")

    data_identifier = f"{mngr.run_phase}_{mngr.dir_input_predict}"

    batch_identifier = str(mngr.check_spread_of_input_error_sample_ts_i_batch)

    if hasattr(mngr, "active_channel_mask"):
        batch_identifier = f"{mngr.calib.active_channel_mask}_{batch_identifier}"

    PDFPlotCollection(mngr, figs=[fig]).save_to_file(
        get_debug_output_filepath(
            mngr,
            os.path.join("input_uncertainties", data_identifier),
            f"ts_relative_std_{batch_identifier}.pdf",
        )
    )

    threshold = 0.5  # warn at realtive standard deviation larger than 50 % of the ts
    n_high_spread = (ts_relative_std > threshold).sum().sum()
    ratio_high_spread = n_high_spread / len(ts_relative_std)
    if ratio_high_spread >= 0.01:
        mngr.log.warn(
            f"Check: Spread of TS for input error samples high for {n_high_spread}/{len(ts_relative_std)} events ({n_high_spread / len(ts_relative_std):.1f}%) "
        )
