from __future__ import annotations

from typing import Dict, List, Literal

from pydantic import PositiveInt, confloat
from typing_extensions import TypedDict

from trans_finder.flares.core.types import ChannelName, MinMaxStr

FluctType = Literal["flat_plateau", "gaussian"]

LongTermStateType = Literal["plateau", "linear_trend"]


class FluctuationPreset(TypedDict):
    # scalar in units of feature standard deviation, positive or negative,
    # key names are irrelevant (e.g. "up", "down")
    amplitude_confs: Dict[str, Dict[MinMaxStr, float]]

    # in units of data_prep_fluctuation_time_unit
    duration: Dict[MinMaxStr, float]

    # only positive, in units of min(data_prep_n_days_per_step)
    temporal_channel_shift: Dict[MinMaxStr, float]

    # types of fluctuation to randomly choose from
    types: List[FluctType]


class LongTermStatePreset(TypedDict):
    # scalar in units of feature standard deviation, positive or negative,
    # key names are irrelevant (e.g. "up", "down")
    amplitude: Dict[MinMaxStr, float]

    # in units of data_prep_fluctuation_time_unit
    duration: Dict[MinMaxStr, float]

    # types of long term state to randomly choose from
    types: List[LongTermStateType]


class FlareTemplateConfig(TypedDict):
    flare_channels: List[ChannelName]
    flare_duration: PositiveInt
    real_scaling_template_name: str
    template_flux_scaling: List[confloat(ge=0)]
    n_template_seeds: PositiveInt

    # duration in days of a flare. has historically been an integer.
    # may work with float, but has not been tested
    systematic_evaluation_durations: List[PositiveInt]

    systematic_evaluation_combinations: Dict[str, List[ChannelName]]
