from __future__ import annotations

from typing import Dict, Tuple

from pydantic import BaseModel, PositiveFloat, conlist

from trans_finder.flares.core.types import ChannelName, RunPhase


class RequiredConfigParams(
    BaseModel,
    protected_namespaces=(),  # disabling the model protected namespace of pydantic due to model_ attributes
    validate_default=True,
):
    # the name of the folder with the input dataset (located in input/flares/)
    input_dataset_version_name: str

    # how may time-steps are used in total
    n_steps_tot: int

    # how may time-steps are used for the decoder (predictions)
    n_steps_dec: int

    # specify a sequence of run_phases, optionally with parameters attached
    pipeline_flow: conlist(Tuple[RunPhase, dict], min_length=1)

    # the interval width of a network step, defined for each data channel
    # in units of days, where it is assumed that the time provided in
    # data_prep_day_column_name is in units of MJD.
    data_prep_n_days_per_step: Dict[ChannelName, PositiveFloat]
