from __future__ import annotations

from typing import Dict, List

from pydantic import BaseModel

from trans_finder.flares.core.tags import TAG_EVT_TYPE
from trans_finder.flares.core.types import EventClass, RunPhase, to_list


class Constants(BaseModel):
    """values here should not be modified at runtime (considered immutable)"""

    tag_model_base: str = "cluster_anom_det"

    # define possible run phases
    valid_run_phases: List[RunPhase] = to_list(RunPhase)
    data_step_prefix: str = "t"
    data_feat_prefix: str = ""

    # defines the formatting of column names in the time series, self.data_prep_time_step_prefix is the prefix
    # used before the number of the current time step, column names will look like 'n_fermi_t_0'
    data_prep_time_step_prefix: str = "t"

    # names and ranges of meta-data columns used as auxiliary variables
    # (the predictions are binned in these dimensions)
    meta_bin_col_params: Dict[str, dict] = {
        TAG_EVT_TYPE: {"type": "list", "values": [to_list(EventClass)]}
    }

    train_phase_0_evt_class_blacklist: List[EventClass] = [
        "signal_dec",
        "signal_enc",
        "signal_enc_dec",
        "signal_old_enc",
        "signal_old_dec",
        "signal_old_enc_dec",
        "bck_fluct",
        "signal_dec_signal_old_enc",
    ]

    dir_input_data_prep_train: str = "input_data_prep"
    dir_input_data_prep_predict: str = "input_data_prep_predict"
    dir_output_prep_train: str = "intermediate_input_shuffled"
    dir_output_data_prep_predict: str = "intermediate_input_predict"
    dir_input_flare_templates: str = "flare_templates"

    # directory with inputs for training
    dir_input_train: str = "intermediate_input_shuffled"

    # name of table in the hdf file, kept to this value for background compatibility
    tag_hdf: str = "ctools"

    # name of ts column in the input DataFrame
    ts_compare: str = "ctools_ts"  # todo delete ?

    # tag in the input DataFrame indicating the event class to filter on
    tag_evt_type: str = TAG_EVT_TYPE
