from trans_finder.flares.data_prep_helpers.channel_padding import (
    create_channel_padding_infer,
)
from trans_finder.flares.data_prep_helpers.fluctuation import (
    create_custom_fluctuation_from_conf,
)


def get_fluctuation_scenarios(mngr, aggregated_data):
    return []


def custom_fluct_scenario(
    mngr,
    aggregated_data,
    scenario_id,
    custom_fluct_confs,
    seed=0,
    add_background_states=False,
):
    realisation = mngr.data_prep.create_realisation(
        aggregated_data,
        shuffling_strategy="step_based",
        do_randomize_temporal_offsets=True,
        do_shift_input_data_in_time=True,
        add_fluctuations=False,
        add_flares=False,
        add_background_states=add_background_states,
        realisation_id=scenario_id,
        realisation_seed=seed,
        channel_padding=create_channel_padding_infer(mngr),
    )

    realisation.custom_name = scenario_id

    assert custom_fluct_confs is not None

    for fluct_id, conf in enumerate(custom_fluct_confs):
        if any(
            tag not in mngr.data_prep_channel_tags
            for tag in conf["channel_start_times"]
        ):
            mngr.log.warn(
                f"custom_fluct_scenario '{scenario_id}': Skipping fluctuation due to non-existing channel name."
            )
            continue

        conf["id"] = fluct_id
        conf["type_name"] = "flat_plateau"

        fluct = create_custom_fluctuation_from_conf(mngr, conf)

        realisation.signal_injections.append(fluct)

    return realisation, {}
