# from line_profiler_pycharm import profile
import pandas as pd

from trans_finder.flares.data_prep_helpers.recreate_aggregated_data import (
    recreate_aggregated_data_from_output_df,
)
from trans_finder.flares.plot_list.input_with_sliding_window_context import (
    plot_inputs_with_sliding_window_context,
)
from trans_finder.flares.plot_list.lightcurve import plot_lightcurve
from trans_finder.flares.plot_list.scenario_highlights import (
    do_plot_scenario_highlights,
)


# @profile
def write_custom_scenario(
    mngr,
    scenario,
    df_batched_hdf_writer,
    do_plot_lightcurve=True,
    time_filter_range=None,
    plot_output_dir=None,
    extra_postprocessing_fn=None,
):
    if plot_output_dir is None:
        plot_output_dir = "sliding_window"

    realisation, metadata_dict = scenario

    print(f"{realisation.id=} {realisation.custom_name=}, {repr(metadata_dict)}")

    network_df = mngr.data_prep.write_realisation(
        realisation,
        yet_desired_evt_classes=[],
        # fixed_evt_class="custom",
        extra_metadata_dict=metadata_dict,
        time_filter_range=time_filter_range,
        extra_postprocessing_fn=extra_postprocessing_fn,
        always_apply_all_injections=True,
    )
    df_batched_hdf_writer.add(network_df)

    if (
        realisation.id not in mngr.plot_inputs_with_sliding_window_context_conf
        and not do_plot_lightcurve
    ):
        return None

    reconstructed_df = recreate_aggregated_data_from_output_df(mngr, network_df)

    do_plot_scenario_highlights(mngr, realisation, reconstructed_df, plot_output_dir)

    if do_plot_lightcurve:
        event_classes = pd.Series(
            network_df[mngr.tag_evt_type].to_numpy(),
            index=network_df["time_right_edge"],
        )
        plot_lightcurve(
            mngr, reconstructed_df, realisation.id, plot_output_dir, event_classes
        )

    if realisation.id in mngr.plot_inputs_with_sliding_window_context_conf:
        scenario_conf = mngr.plot_inputs_with_sliding_window_context_conf[
            realisation.id
        ]

        plot_inputs_with_sliding_window_context(
            mngr,
            network_df,
            reconstructed_df,
            realisation,
            temporal_offset_fixed_min=scenario_conf.get(
                "temporal_offset_ylim_min", None
            ),
            show_shuffling_window=scenario_conf.get("show_shuffling_window", False),
            show_signal_injection=scenario_conf.get("show_signal_injection", False),
        )

    return reconstructed_df
