from __future__ import annotations

from typing import TYPE_CHECKING, Dict

import pandas as pd

if TYPE_CHECKING:
    from trans_finder.flares.manager import Manager


def recreate_aggregated_data_from_output_df(
    mngr: Manager, network_df, with_errors=True
) -> Dict[str, pd.DataFrame]:
    assert network_df["time_right_edge"].is_monotonic_increasing
    assert network_df["time_right_edge"].is_unique

    time_col = mngr.data_prep_day_column_name

    recreated_data = {}

    for channel_tag in mngr.data_prep_channel_tags:
        relevant_step_features = mngr.data_prep_error_feature_cols[channel_tag]

        if with_errors:
            relevant_step_features = (
                mngr.data_prep_error_feature_cols[channel_tag]
                + mngr.data_prep_error_up_cols[channel_tag]
                + mngr.data_prep_error_down_cols[channel_tag]
            )

        relevant_features = []

        recreated_data[channel_tag] = pd.DataFrame()

        for timeseries_idx, timeseries in enumerate(network_df.iterrows()):
            if timeseries_idx == 0:
                step_range = range(mngr.n_steps_tot)
            else:
                step_range = [mngr.n_steps_tot - 1]

            for t in step_range:
                step_time = (
                    network_df.loc[timeseries_idx, f"time_right_edge"]
                    + network_df.loc[
                        timeseries_idx,
                        f"{channel_tag}_t_{t}_{mngr.tag_temporal_offset}",
                    ]
                )

                recreated_data[channel_tag].loc[step_time, time_col] = step_time

                for ft_name in relevant_step_features:
                    recreated_data[channel_tag].loc[step_time, ft_name] = (
                        network_df.loc[timeseries_idx, f"{channel_tag}_t_{t}_{ft_name}"]
                    )
                for ft_name in relevant_features:
                    recreated_data[channel_tag].loc[step_time, ft_name] = (
                        network_df.loc[timeseries_idx, ft_name]
                    )

        assert recreated_data[channel_tag].index.is_monotonic_increasing
        assert recreated_data[channel_tag].index.is_unique

    return recreated_data
