from keras import Input
from keras.layers import Dense, Dropout, Flatten
from keras.models import Model
from tensorflow import keras

from trans_finder.flares.model_utils.lstm_vae_model import Sampling
from trans_finder.flares.model_utils.utils import plot_tf_model_overview


def build_encoder(mngr, input_shape):
    latent_dim = mngr.model_learned_representation_n_dim
    epsilon_std = 1.0

    encoder_input = Input(
        shape=input_shape,
        name="input_encs",
    )

    x = encoder_input

    # forecast = x[..., : mngr.n_features]
    # real_data = x[..., mngr.n_features :]
    #
    # forecast_without_offsets = RemoveTemporalOffsetsLayer(mngr)(forecast)
    #
    # x = tf.concat([forecast_without_offsets, real_data], -1)

    x = Flatten()(x)

    for i_size in range(len(mngr.autoenc_enc_sizes)):
        lstm_size = mngr.autoenc_enc_sizes[i_size]

        x = Dense(
            lstm_size,
            activation="relu",
            # return_sequences=return_sequences,
            # dropout=mngr.rnn_dropout,
            # recurrent_dropout=0,
            kernel_regularizer=keras.regularizers.L2(mngr.regularizer_l2_scale),
            name=f"encoder_dense_{i_size}",
        )(x)
        x = Dropout(mngr.rnn_dropout)(x)

    z_mu = Dense(latent_dim)(x)
    z_log_var = Dense(latent_dim)(x)

    z = Sampling()([z_mu, z_log_var])

    encoder = Model(encoder_input, [z_mu, z_log_var, z], name="encoder")

    if mngr.log.get_log_level() == "DEBUG" and mngr.run_phase == "train":
        mngr.log.debug([["p", " - encoder.summary():"]])
        encoder.summary()
        plot_tf_model_overview(mngr, encoder, "autoenc_encoder_dense")
    return encoder


def build_decoder(mngr, output_shape):
    latent_dim = mngr.model_learned_representation_n_dim

    decoder_input = Input(
        shape=(latent_dim,),
        name="latent_z",
    )

    x = decoder_input
    # x = RepeatVector(output_shape[0])(x)

    for n_size in range(len(mngr.autoenc_dec_sizes)):
        lstm_size = mngr.autoenc_dec_sizes[n_size]

        x = Dense(
            lstm_size,
            activation="relu",
            # dropout=mngr.rnn_dropout,
            kernel_regularizer=keras.regularizers.L2(mngr.regularizer_l2_scale),
            name=f"decoder_dense_{n_size}",
        )(x)
        x = Dropout(mngr.rnn_dropout)(x)

    x = keras.layers.Dense(output_shape[0] * output_shape[1])(x)
    x = keras.layers.Reshape(output_shape)(x)

    # shape_without_offsets = (
    #     mngr.n_steps_dec,
    #     (2 * mngr.n_features) - len(mngr.temporal_offset_column_indices),
    # )
    # x = keras.layers.Dense(shape_without_offsets[0] * shape_without_offsets[1])(x)
    # x = keras.layers.Reshape(shape_without_offsets)(x)
    #
    # forecast_without_offsets = x[..., : -mngr.n_features]
    # real_data = x[..., -mngr.n_features :]
    #
    # add_offsets_back_layer = AddInputOffsetsBack(mngr)
    # forecast = add_offsets_back_layer([forecast_without_offsets, real_data])
    # x = tf.concat([forecast, real_data], -1)

    decoder = Model(decoder_input, x, name="decoder")

    if mngr.log.get_log_level() == "DEBUG" and mngr.run_phase == "train":
        mngr.log.debug([["p", " - decoder.summary():"]])
        decoder.summary()
        plot_tf_model_overview(mngr, decoder, "autoenc_decoder_dense")
    return decoder
