import tensorflow as tf
from tensorflow import keras


class AddInputOffsetsBack(keras.layers.Layer):
    def __init__(self, n_features, temporal_offset_column_indices):
        super(AddInputOffsetsBack, self).__init__()
        self.feature_indices_without_offsets = [
            i for i in range(n_features) if i not in temporal_offset_column_indices
        ]
        self.temporal_offset_column_indices = temporal_offset_column_indices

    def call(self, inputs):
        output_prob_sample, input_decs = inputs

        outputs = [i for i in range(input_decs.shape[-1])]

        for i_feature, i_output in enumerate(self.feature_indices_without_offsets):
            outputs[i_output] = output_prob_sample[..., i_feature]

        for i_output in self.temporal_offset_column_indices:
            outputs[i_output] = input_decs[..., i_output]

        return tf.stack(outputs, axis=-1)


class RemoveTemporalOffsetsLayer(keras.layers.Layer):
    def __init__(self, n_features, temporal_offset_column_indices):
        super(RemoveTemporalOffsetsLayer, self).__init__()
        self.feature_indices_without_offsets = [
            i for i in range(n_features) if i not in temporal_offset_column_indices
        ]

    def call(self, inputs):
        return tf.stack(
            [inputs[..., i] for i in self.feature_indices_without_offsets], axis=-1
        )
