import numpy as np
import pandas as pd
import tensorflow as tf

from trans_finder.flares.helpers_debugging import get_debug_output_filepath


def get_batch_size(mngr, force_predict_size=False):
    if force_predict_size:
        return mngr.predict_batch_size
    elif mngr.run_phase == "train":
        return mngr.train_batch_size
    elif mngr.run_phase == "calib":
        return mngr.calib_batch_size
    elif mngr.run_phase in [
        "predict",
        "debug_keras_model",
        "bgmm_fit",
        "inspect_inputs",
    ]:
        return mngr.predict_batch_size
    else:
        raise ValueError("Unknown run_phase in get_batch_size()")


def add_scaled_loss(self, loss, name, weight):
    scaled_loss = tf.scalar_mul(weight, loss)
    self.add_loss(scaled_loss)
    # self.add_metric(weight, name=f"weight_loss_{name}")

    # self.add_metric(
    #     loss,
    #     name=f"loss_{name}",
    #     aggregation="mean",
    # )

    self.add_metric(
        scaled_loss,
        name=f"scaled_loss_{name}",
        aggregation="sum",
    )

    return


def dataset_to_csv(tf_dataset, fname):
    np_list = np.concatenate(
        [e[0]["enc"] for e in list(tf_dataset.as_numpy_iterator())]
    )
    flattened = np_list.reshape(np_list.shape[0], -1)
    df = pd.DataFrame(flattened)
    df.insert(0, "class", 0)
    df.to_csv(fname, header=None, index=False)

    return


def plot_tf_model_overview(mngr, model, name):
    try:
        if mngr.log.get_log_level() == "DEBUG" and mngr.run_phase == "train":
            dot_img_file = get_debug_output_filepath(mngr, "keras_model", f"{name}.png")
            tf.keras.utils.plot_model(model, to_file=dot_img_file, show_shapes=True)
    except Exception:
        pass


def fn_rmse_batch(truth, prediction):
    rmse = tf.sqrt(tf.reduce_mean(tf.square(tf.subtract(truth, prediction))))
    return rmse


def fn_rmse_samples(truth, prediction):
    rmse = tf.sqrt(
        tf.reduce_mean(tf.square(tf.subtract(truth, prediction)), axis=[1, 2])
    )
    return rmse
