"""This module tries to load comet_ml if installed. When comet_ml is not installed,
initialize_experiment() returns an alternative object that silently consumes any method calls,
ensuring code compatibility, both with an without comet_ml installed."""

try:
    from comet_ml import Experiment as CometExperiment
except ImportError:
    CometExperiment = None


class MockCometExperiment:
    def __getattr__(self, name):
        def catchall(*args):
            pass

        return catchall


def initialize_experiment(run_phase, project_tag, **kwargs):
    # Comet will be only activated in training stage and when the module is installed
    is_comet_active = run_phase == "train" and CometExperiment is not None

    if is_comet_active:
        return CometExperiment(
            project_name=f"trans_finder_{project_tag}",
            log_code=False,
            **kwargs,
        )

    else:
        return MockCometExperiment()
