import numpy as np
import pandas as pd
import seaborn as sns

from trans_finder.flares.helpers_debugging import denormalize_model_output
from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def autoenc_create_all_plots_from_model_outputs(
    mngr, model_outputs, out_file_name, labels=None
):
    auto_enc_hidden = model_outputs[mngr.tag_cluster_space_sample]

    # denormed_reconstruction_mean_abs_error
    denormed_input = denormalize_model_output(
        mngr, model_outputs[mngr.tag_auto_enc_input]
    )
    denormed_output = denormalize_model_output(
        mngr, model_outputs[mngr.tag_auto_enc_reconstruction]
    )

    absolute_error = np.abs(denormed_input - denormed_output)
    denormed_reconstruction_error = absolute_error.reshape(
        absolute_error.shape[0], -1
    ).mean(axis=1)

    auto_enc_reconstruction_error = model_outputs[
        mngr.tag_auto_enc_reconstruction_error
    ]

    autoenc_create_all_plots(
        mngr,
        auto_enc_hidden,
        auto_enc_reconstruction_error,
        denormed_reconstruction_error,
        out_file_name,
        labels,
    )


def autoenc_create_all_plots(
    mngr,
    hidden_repr,
    reconstruction_error,
    denormed_reconstruction_error,
    out_file_name,
    labels=None,
):
    pdfplots = PDFPlotCollection(mngr)

    # data_auto_enc_input = auto_enc_input.reshape(
    #     auto_enc_input.shape[0], -1
    # )
    # label_auto_enc_input = "autoencoder input (forecast_bck + d1)"

    # make a plot of the principal components in the autoencoder input
    # as a baseline for comparison

    plot_df = pd.DataFrame(
        hidden_repr, columns=[f"latent dim {i}" for i in range(hidden_repr.shape[1])]
    )
    plot_df["label"] = labels

    if mngr.model_learned_representation_n_dim == 1:
        #   Histogram of the hidden repr
        fig, ax = pdfplots.create_figure()
        pdfplots.make_histogram(ax, hidden_repr)
        ax.set_title("latent representations")

    elif mngr.model_learned_representation_n_dim == 2:
        # g = sns.PairGrid(plot_df, height=6)
        # g.map_upper(sns.histplot)
        # g.map_lower(sns.kdeplot, fill=True)
        # g.map_diag(sns.histplot, kde=True)

        g = sns.JointGrid(
            data=plot_df, x="latent dim 0", y="latent dim 1", hue="label", height=10
        )
        g.plot_joint(sns.scatterplot, marker="x")
        g.plot_marginals(sns.histplot, fill=True)
        g.refline(x=0, y=0)

        pdfplots.figs.append(g.fig)

    elif mngr.model_learned_representation_n_dim > 2:
        if mngr.run_phase != "train":
            try:
                g = sns.PairGrid(plot_df, diag_sharey=False, height=5, hue="label")
                g.map_upper(sns.scatterplot, marker="x")
                g.map_lower(sns.kdeplot)
                g.map_diag(sns.kdeplot)
                g.add_legend()

                pdfplots.figs.append(g.fig)
            except Exception as e:
                print(e)

    g = sns.displot(x=reconstruction_error, hue=labels, element="step")
    g.set(xlabel="keras autoencoder reconstruction error")
    pdfplots.figs.append(g.fig)

    g = sns.displot(x=denormed_reconstruction_error, hue=labels, element="step")
    g.set(xlabel="mean absolute reconstruction error on denormalized data")
    pdfplots.figs.append(g.fig)

    # fig, ax = pdfplots.create_figure()
    # pdfplots.make_histogram(ax, reconstruction_error, labels=labels, log_scale_x=False)
    # ax.set_title("keras autoencoder reconstruction error")
    #
    # fig, ax = pdfplots.create_figure()
    # pdfplots.make_histogram(ax, reconstruction_error, labels=labels)
    # ax.set_title("keras autoencoder reconstruction error")

    # fig, ax = pdfplots.create_figure()
    # pdfplots.make_histogram(
    #     ax, denormed_reconstruction_error, labels=labels, log_scale_x=False
    # )
    # ax.set_title("mean absolute reconstruction error on denormalized data")
    #
    # fig, ax = pdfplots.create_figure()
    # pdfplots.make_histogram(ax, denormed_reconstruction_error, labels=labels)
    # ax.set_title("mean absolute reconstruction error on denormalized data")
    #
    pdfplots.save_to_file(out_file_name)
