import pandas as pd

from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def make_feature_distribution_plots(
    mngr,
    np_array,
    file_name,
):
    pdfplots = PDFPlotCollection(mngr)
    n_features = np_array.shape[-1]

    for n_feature in range(n_features):
        fig, ax = pdfplots.create_figure()

        pdfplots.make_histogram(ax, np_array[..., n_feature])

        ax.set_title(f"Feature {n_feature}")

    pass

    pdfplots.save_to_file(file_name)


def make_superimposed_feature_dist_timesteps_plot(
    mngr,
    np_array,
    file_name,
    vertical_figsize_per_timestep=2,
    horizontal_figsize=6,
):
    pdfplots = PDFPlotCollection(mngr)

    n_timesteps = np_array.shape[1]

    fig, spec = pdfplots.create_figure_grid(
        n_timesteps,
        1,
        opts={
            "fig_size": (
                horizontal_figsize,
                int(vertical_figsize_per_timestep * n_timesteps),
            ),
        },
    )

    fig_title = (
        "feature value distribution"
        "\nwith one row being one time step and"
        "\nsuperimposed feature columns for each time step"
    )

    try:
        firstax = True
        sharedax = None
        for timestep in range(n_timesteps):
            ax = pdfplots.get_figure_grid_axis(fig, spec, timestep, 0)

            if firstax:
                ax.set_title(fig_title, pad=6)
                firstax = False

            if sharedax is None:
                sharedax = ax
            else:
                ax.sharex(sharedax)
                ax.sharey(sharedax)

            ax.tick_params(
                axis="x",  # changes apply to the x-axis
                which="both",  # both major and minor ticks are affected
                bottom=True,
                top=False,
                labelbottom=True,
            )

            timestep_data = np_array[:, timestep, :]
            df = pd.DataFrame(timestep_data)
            df.plot.kde(ax=ax, bw_method=0.01)

        pdfplots.save_to_file(file_name)

    except Exception as e:
        # one source of error: singular value in a feature, no variance at all, this leads to an error in the kde
        print("Error when plotting: ")
        print(e)
        pass
