import os

import numpy as np
import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt

from trans_finder.flares.helpers_debugging import get_debug_output_filepath
from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def create_plot(
    mngr,
    plot_df,
    flux_label,
    significance_label,
    cta_sparsity_label,
    grouped_by_sparsity,
):
    fig, ax = plt.subplots(
        nrows=1,
        ncols=1,
        # figsize=(25, 4),
        constrained_layout=True,
    )

    ax.grid(
        **{
            "color": "#4f8c9d",
            "alpha": 0.8,
            "linestyle": ":",
            "linewidth": 0.5,
            # "axis": "x",
        }
    )

    def add_errors(grp):
        grp["upper"] = grp[significance_label].quantile(0.68)
        grp["lower"] = grp[significance_label].quantile(0.32)
        return grp

    if grouped_by_sparsity:
        sns.lineplot(
            data=plot_df,
            ax=ax,
            x=flux_label,
            y=significance_label,
            hue=cta_sparsity_label if grouped_by_sparsity else None,
            style=cta_sparsity_label if grouped_by_sparsity else None,
            # ci=1 ,
            ci=None,
            palette=None if not grouped_by_sparsity else "tab10",
        )

        for idx, (sparsity, df_group) in enumerate(plot_df.groupby(cta_sparsity_label)):
            df_group = df_group.groupby(flux_label).apply(add_errors)

            ax.fill_between(
                x=df_group[flux_label],
                y1=df_group["upper"],
                y2=df_group["lower"],
                alpha=0.2,
                facecolor=ax.lines[idx].get_color(),
                linewidth=0.0,
            )
    else:
        sns.lineplot(
            data=plot_df,
            ax=ax,
            x=flux_label,
            y=significance_label,
            hue=cta_sparsity_label if grouped_by_sparsity else None,
            style=cta_sparsity_label if grouped_by_sparsity else None,
            # ci=1 ,
            ci=None,
            palette=None if not grouped_by_sparsity else "tab10",
        )

        # plt.plot(plot_df[significance_label], plot_df[flux_label])

        df_group = plot_df.groupby(flux_label).apply(add_errors)
        ax.fill_between(
            x=df_group[flux_label],
            y1=df_group["upper"],
            y2=df_group["lower"],
            alpha=0.2,
        )

    ax.ticklabel_format(style="sci", scilimits=(0, 0))

    scale_1_template_path = os.path.join(
        mngr.tag_input_dir,
        mngr.dir_input_flare_templates,
        mngr.plot_flare_significance_over_template_flux["real_scaling_template_name"],
    )
    flux_real_scale = pd.read_hdf(scale_1_template_path)["sim_flux_input"].iat[0]
    df_real_flux = plot_df[flux_label].loc[
        np.abs(plot_df[flux_label] - flux_real_scale).argmin()
    ]
    print(flux_real_scale)

    mean_significance_at_reference = plot_df[plot_df[flux_label] == df_real_flux][
        significance_label
    ].mean()

    plt.title(
        f"mean significance at reference flux: {mean_significance_at_reference:.3f}"
    )

    plt.axvline(x=flux_real_scale, c="grey", linestyle="--")

    out_file_name = get_debug_output_filepath(
        mngr,
        "flare_significance_over_template_flux",
        f"plot{'_grouped_by_sparsity' if grouped_by_sparsity else ''}.pdf",
    )

    PDFPlotCollection(mngr, [fig]).save_to_file(out_file_name)

    pass
