import numpy as np
from matplotlib import colors

from trans_finder.flares.plot_list.util.plot_config import gridlines_config, tick_params


def plot_lightcurve_combined_normalized(
    mngr,
    reconstructed_dfs,
    ax,
):
    realisation_data = reconstructed_dfs

    marker_groups = list(realisation_data.keys())

    group_colors = {
        group_name: mngr.plot_lib.get_plot_style(
            index=1 + idx,  # offset to not have the pink
            tag="main_data",
            style_set="main_0",
            has_line=False,
            has_marker=True,
        )["color"]
        for idx, group_name in enumerate(mngr.channel_groups.keys())
    }

    def get_channel_group(channel_tag):
        """gets the first channel group containing the passed channel tag"""
        for group, channel_tag_list in mngr.channel_groups.items():
            if channel_tag in channel_tag_list:
                return group
        raise ValueError(channel_tag)

    shift = 0
    previous_channel_group = None

    y_gridlines = []
    for channel_tag, channel_df in realisation_data.items():
        channel_group = get_channel_group(channel_tag)
        if previous_channel_group is None:
            previous_channel_group = channel_group

        plot_style = mngr.plot_lib.get_plot_style(
            index=marker_groups.index(channel_tag),
            tag="main_data",
            style_set="main_0",
            has_line=False,
            has_marker=True,
        )
        if "alpha" in plot_style:
            del plot_style["alpha"]
        plot_style.update(
            dict(
                color=colors.to_rgba(group_colors[channel_group], 1.0),
                ecolor=colors.to_rgba(group_colors[channel_group], 0.6),
                # linewidth=1,
                # capsize=5,
                # capthick=0.1,
                # fmt="o",
                markersize=4,
                # ls="none",
            )
        )

        y = channel_df[mngr.data_prep_error_feature_cols[channel_tag]].to_numpy()
        x = channel_df[mngr.data_prep_day_column_name].to_numpy()
        error_up = channel_df[mngr.data_prep_error_up_cols[channel_tag]].to_numpy()
        error_down = channel_df[mngr.data_prep_error_down_cols[channel_tag]].to_numpy()

        def transform(data_space):
            transformed = np.squeeze(data_space)
            return transformed

        y = transform(y)
        error_up = transform(error_up)
        error_down = transform(error_down)

        norm = np.std(y)
        error_up = error_up / norm
        error_down = error_down / norm

        y = y / norm

        # center at 0
        y = y - np.median(y)

        # shift += 3
        shift -= 10

        if previous_channel_group != channel_group:
            shift -= 5
        previous_channel_group = channel_group
        y += shift

        y_gridlines.append(shift)

        label: str = mngr.channel_labels[channel_tag]

        yerr = np.stack((error_down, error_up))

        ax.errorbar(
            y=y,
            x=x,
            yerr=yerr,
            label=label,
            **plot_style,
        )

    ax.set_yticks(y_gridlines)
    ax.grid(**gridlines_config)
    ax.tick_params(**tick_params)
    ax.legend()
    x_label = "Time [days]"
    # y_label = r"log$_{10}$( Flux $[$ photons cm$^{-2}$ s$^{-1}$ ] )"
    y_label = r"Normalized Brightness"
    ax.set_xlabel(x_label)
    ax.set_ylabel(y_label)
    ax.yaxis.set_ticklabels([])
    # add_channel_legend(ax, mngr)

    return ax
