import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import kde
from umap import UMAP


def create_projection_plot_twosets(reducer, arr_contours, arr_points, ax):
    # Concatenate the arrays for fitting
    concat_arr = np.concatenate([arr_contours, arr_points])

    # Fit and transform the concatenated array
    embeddings = reducer.fit_transform(concat_arr)

    # Separate the embeddings for arr1 and arr2
    embedding1 = embeddings[: arr_contours.shape[0]]
    embedding2 = embeddings[arr_contours.shape[0] :]

    # Create a density estimator for the contour plot
    k = kde.gaussian_kde(embedding1.T)
    xi, yi = np.mgrid[
        embedding1[:, 0].min() : embedding1[:, 0].max() : 100j,
        embedding1[:, 1].min() : embedding1[:, 1].max() : 100j,
    ]
    zi = k(np.vstack([xi.flatten(), yi.flatten()]))

    # Draw points for arr2
    ax.scatter(
        embedding2[:, 0], embedding2[:, 1], c="blue", marker="x", s=5, alpha=0.75
    )

    # Draw the contour plot for arr1
    ax.contour(xi, yi, zi.reshape(xi.shape), colors="k")


if __name__ == "__main__":
    n_rows = 10000
    n_dims = 20

    arr1 = np.random.rand(n_rows, n_dims) * 1.5
    arr2 = np.random.rand(n_rows, n_dims)

    reducer = UMAP()
    fig, ax = plt.subplots()
    create_projection_plot_twosets(reducer, arr1, arr2, ax)
    plt.show()
