import matplotlib.pyplot as plt
import numpy as np

from trans_finder.flares.plot_list.util.plot_config import gridlines_config, tick_params


def residual_violin_plot(ax, actual, predicted, labels):
    # Calculate residuals
    residuals = actual - predicted

    # Create violin plot
    parts = ax.violinplot(residuals, showmedians=True)

    # Color the body of the violin plot
    for pc in parts["bodies"]:
        pc.set_facecolor("blue")
        pc.set_edgecolor("black")
        pc.set_alpha(0.7)

    # Color the median, min and max
    parts["cmedians"].set_edgecolor("red")
    parts["cmins"].set_edgecolor("green")
    parts["cmaxes"].set_edgecolor("orange")

    # Adding labels
    plt.xticks(np.arange(1, len(labels) + 1), labels)
    plt.ylabel("Residuals")

    # Extend the y-axis limits of your plot by 10% up and down
    ymin, ymax = ax.get_ylim()
    yrange = ymax - ymin
    extension = yrange * 0.1
    ax.set_ylim(ymin - extension, ymax + extension)

    # Add a label just above each maximum and below each minimum
    for i in range(residuals.shape[1]):
        max_value = np.max(residuals[:, i])
        ax.text(i + 1, max_value, f"{max_value:.2e}", ha="center", va="bottom")

        min_value = np.min(residuals[:, i])
        ax.text(i + 1, min_value, f"{min_value:.2e}", ha="center", va="top")

    ax.grid(**gridlines_config)
    ax.tick_params(**tick_params)
    #
    # ax.set_yscale("log")


if __name__ == "__main__":
    # Create a figure and a set of subplots
    fig, ax = plt.subplots()

    # Example usage
    actual = np.random.rand(100, 5)
    predicted = np.random.rand(100, 5)
    labels = ["feature1", "feature2", "feature3", "feature4", "feature5"]
    residual_violin_plot(ax, actual, predicted, labels)

    plt.show()
