import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as stats

from trans_finder.flares.core.tags import TAG_MEDIAN, TAG_SIGNIFICANCE_CLUSTER_SPACE
from trans_finder.flares.helpers_debugging import get_debug_output_filepath
from trans_finder.flares.plot_list.util.plot_config import gridlines_config, tick_params
from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def plot_significance_squared_dist_for_each_custom_realisation(mngr, df_diagnostics):
    for realisation_id in df_diagnostics.realisation_id.unique():
        realisation_df = df_diagnostics[df_diagnostics.realisation_id == realisation_id]

        plot_significance_squared_distribution(
            mngr, realisation_df, f"custom_realisation_{realisation_id}"
        )


def plot_significance_squared_distribution(mngr, df_diagnostics, data_desc):
    significance_col = "_".join([TAG_SIGNIFICANCE_CLUSTER_SPACE, TAG_MEDIAN])
    selection_df = df_diagnostics
    # bck_pure_df = df_diagnostics[df_diagnostics["evt_class"] == "bck_pure"]

    sig_squared = selection_df[significance_col] ** 2

    # axis properties
    x_width = 0.25
    x_min = 0
    x_n_bins = int(((sig_squared.max() - x_min) / x_width))
    x_n_bins = max(x_n_bins, 10)
    x_max = x_min + x_n_bins * x_width
    x_max = max(x_max, 1)
    # print('out_vals.max()',tag_now,out_vals.min(),out_vals.max())

    # normalise, so that the pdf will also be the
    # probabiity mass function [--> integral==sum==1]
    out_wgts = np.ones_like(sig_squared) / (len(sig_squared) * x_width)

    x = np.linspace(x_min, x_max, x_n_bins + 1)
    y, _ = np.histogram(
        sig_squared,
        weights=out_wgts,
        bins=x,
    )

    fig, ax = plt.subplots(
        nrows=1,
        ncols=1,
    )

    x_mid = [x[n] + 0.5 * (x[n + 1] - x[n]) for n in range(len(x) - 1)]

    plot_style = mngr.plot_lib.get_plot_style(
        index=0,
        tag="main_data",
        style_set="autumn",
    )
    # label = mngr.plots.get_title(mngr)

    ax.plot(
        x_mid,
        y,
        **plot_style,
        # label=label,
    )

    # add superimposed chi-square
    n_tag_chi_2 = 1
    x_max_chi_2 = ax.get_xlim()[1]
    x = np.linspace(x_min, x_max_chi_2, x_n_bins + 1)

    # at this point we only compare to ndf=1, since even if
    # [chi_2_ndf_ctools>1] above, we convert the p-values.
    y = stats.norm.sf(np.sqrt(x))
    # y = 0.5 * stats.chi2.sf(x, df=1)

    plot_style = mngr.plot_lib.get_plot_style(
        index=n_tag_chi_2,
        tag="main_data",
        style_set="autumn",
    )
    # label = mngr.plots.get_title(mngr.tag_chi_2)

    ax.plot(
        x,
        y,
        **plot_style,
        # label=label,
    )

    ax.set_yscale("log")
    ax.grid(**gridlines_config)
    ax.tick_params(**tick_params)
    x_label = r"$\sigma^{2}$"
    y_label = r"1/N dN/d$\sigma^{2}$"
    ax.set_xlabel(x_label)
    ax.set_ylabel(y_label)
    ax.set_title(data_desc)

    PDFPlotCollection(mngr, [fig]).save_to_file(
        get_debug_output_filepath(
            mngr, significance_col + "_squared_dist", f"{data_desc}.pdf"
        )
    )
    #
    #
