import numpy as np
import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt

from trans_finder.flares.utilities.pdf_plot_collection import PDFPlotCollection


def do_hist_no_labels(
    histogram_data: np.array, data_desc: str, labels: pd.Series = None
):
    if labels is None:
        label = "all combined"
    else:
        label = labels[0]

    fig, ax = plt.subplots(
        nrows=1,
        ncols=1,
        figsize=(10, 6),
    )

    g = sns.histplot(
        ax=ax,
        data={label: histogram_data},
        legend=labels is not None,
        stat="percent",
        common_norm=False,
        element="step",
    )
    ax.set_xlabel(data_desc)
    ax.set_yscale("log")
    # ax.set_xlim(left=left_limit)

    return fig


def do_detailed_histograms_grouped_by_label(
    mngr,
    histogram_data: np.array,
    data_desc: str,
    out_file_name: str,
    labels: pd.Series = None,
    vertical_lines=None,
):
    unique_labels = labels.unique()

    figs = []

    if labels is None or len(unique_labels) == 1:
        figs.append(do_hist_no_labels(histogram_data, data_desc, labels))

    else:
        n_plots = 1 + len(unique_labels)

        fig, axes = plt.subplots(
            nrows=n_plots,
            ncols=1,
            sharex="all",
            # sharey="all",
            figsize=(15, 5 * n_plots),
        )
        figs.append(fig)

        data = {}
        if labels is None:
            data["all combined"] = histogram_data
        else:
            for label in unique_labels:
                data[label] = histogram_data[labels == label]

        # all combined
        g = sns.histplot(
            ax=axes[0],
            data=data,
            legend=labels is not None,
            stat="percent",
            common_norm=False,
            element="step",
        )
        axes[0].set_xlabel(data_desc)
        axes[0].xaxis.set_tick_params(which="both", labelbottom=True)
        axes[0].set_yscale("log")
        # axes[0].set_xlim(left=left_limit)

        if vertical_lines is not None:
            for label, x_pos in vertical_lines.items():
                if x_pos is None:
                    mngr.log.info(
                        [
                            [
                                "b",
                                f"Encountered None in vertical line '{label}'. Skipping it.",
                            ]
                        ]
                    )
                    continue
                axes[0].axvline(x=x_pos, c="black", linewidth=0.5)

        for i, label in enumerate(unique_labels):
            if len(data[label]) == 0:
                continue
            g = sns.histplot(
                ax=axes[1 + i],
                data={label: data[label]},
                legend=labels is not None,
                stat="percent",
                common_norm=False,
                element="step",
            )
            axes[1 + i].set_xlabel(data_desc)
            axes[1 + i].xaxis.set_tick_params(which="both", labelbottom=True)
            axes[1 + i].set_yscale("log")
            # axes[1 + i].set_xscale("log")
            # axes[1 + i].set_xlim(left=left_limit)

    PDFPlotCollection(mngr, figs).save_to_file(out_file_name)
