from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Optional

from trans_finder.flares.exceptions import CalibFileMissing

if TYPE_CHECKING:
    from trans_finder.flares.manager import Manager


class StateManager:
    def __init__(
        self,
        mngr,
        state_id: str,
        dir_tag: Optional[str] = None,
    ):
        """
        A utility to save and load  some state as a joblib file using the mngr.
        :param mngr:
        :param state_id:
        """

        self.mngr: Manager = mngr
        self.state_id = state_id
        self.file_name = f"{self.state_id}_state"
        self.dir_tag = dir_tag if dir_tag is not None else self.mngr.tag_model_dir

    def load_state(self):
        if self.mngr is not None:
            return self.mngr.data.load_calib_file(
                self.dir_tag,
                tag_file=self.file_name,
                tag_prefix=None,
            )

        else:
            return None

    def has_state(self):
        if self.mngr is None:
            return False

        try:
            self.mngr.data.load_calib_file(
                self.dir_tag,
                tag_file=self.file_name,
                tag_prefix=None,
                # is_verb=False,
            )
            return True
        except CalibFileMissing:
            return False
        except Exception as e:
            return False

    def save_state(self, state):
        if self.mngr is not None:
            # ensure output dir exists as the joblib.dump method fails otherwise
            Path(self.mngr.data.get_dir_name(self.dir_tag)).mkdir(
                parents=True, exist_ok=True
            )
            self.mngr.data.save_calib_file(
                self.dir_tag,
                self.file_name,
                state,
                tag_prefix=None,
            )
        else:
            print(f"Skipped saving {self.file_name}")
