"""This module tries to load mlflow if installed. When comet_ml is not installed,
initialize_run() returns an alternative object that silently consumes any method calls,
ensuring code compatibility, both with an without comet_ml installed."""

try:
    import mlflow
except ImportError:
    mlflow = None


class MockObject:
    def __getattr__(self, name):
        def catchall(*args, **kwargs):
            return self

        return catchall


def initialize_mlflow(mngr):
    # Comet will be only activated in training stage and when the module is installed
    is_mlflow_available = mngr.run_phase == "train" and mlflow is not None

    if is_mlflow_available:
        return mlflow

    else:
        return MockObject()
