import pandas as pd

from trans_finder.flares.core.tags import (
    TAG_LOWERQUANT,
    TAG_MEDIAN,
    TAG_SIGNIFICANCE_COMBINED,
    TAG_UPPERQUANT,
)


def make_result_table(mngr, df_diagnostics, add_masked_groups=True):
    assert len(df_diagnostics["realisation_id"].unique()) == 1

    col_sigma = "_".join([TAG_SIGNIFICANCE_COMBINED, TAG_MEDIAN])
    col_sigma_up = "_".join([TAG_SIGNIFICANCE_COMBINED, TAG_UPPERQUANT])
    col_sigma_down = "_".join([TAG_SIGNIFICANCE_COMBINED, TAG_LOWERQUANT])

    # differentiate complete inference, and inference with mask groups for data subsets
    filters = [("", df_diagnostics["input_mask"] == "full")]

    if add_masked_groups:
        filters += [
            (name + "_", df_diagnostics["input_mask"] == name)
            for name in mngr.channel_groups.keys()
        ]

    df = None

    for prefix, filter_condition in filters:
        filtered_df = df_diagnostics[filter_condition].reset_index()

        if df is None:
            df = pd.DataFrame(
                dict(
                    t=filtered_df["time_right_edge"],
                    # t_original=df_diagnostics["time_right_edge"] + mngr.data_prep_analysis_min_time,
                    **{},
                )
            )

            data_prep_analysis_min_time_bin_width = min(
                mngr.data_prep_n_days_per_step[channel_tag]
                for channel_tag in mngr.data_prep_channel_tags
            )
            if data_prep_analysis_min_time_bin_width == 1:
                df["t"] = df["t"].astype(int)

        assert len(df) == len(filtered_df)
        assert (filtered_df["time_right_edge"] == df["t"]).all()
        assert filtered_df["time_right_edge"].is_unique

        df = df.assign(
            **{
                prefix + "sigma": filtered_df[col_sigma],
                prefix + "sigma_high": filtered_df[col_sigma_up],
                prefix + "sigma_low": filtered_df[col_sigma_down],
            }
        )

    df = df.sort_values(by="t")

    return df
