import io

import tensorflow as tf
from matplotlib import pyplot as plt


def tensorboard_log_figure(mngr, fig: plt.Figure, label: str, step: int = 0):
    log_dir = mngr.train.get_tensorboard_params()["log_dir"]
    file_writer = tf.summary.create_file_writer(log_dir)

    def plot_to_image(figure: plt.Figure):
        """Converts the matplotlib plot specified by 'figure' to a PNG image and
        returns it."""
        # Save the plot to a PNG in memory.
        buf = io.BytesIO()
        plt.savefig(buf, format="png")
        buf.seek(0)
        # Convert PNG buffer to TF image
        image = tf.image.decode_png(buf.getvalue(), channels=4)
        # Add the batch dimension
        image = tf.expand_dims(image, 0)
        return image

    # Convert to image and log
    with file_writer.as_default():
        tf.summary.image(label, plot_to_image(fig), step=step)


def tensorboard_log_scalar(mngr, scalar, label, step=0):
    log_dir = mngr.train.get_tensorboard_params()["log_dir"]
    file_writer = tf.summary.create_file_writer(log_dir)
    with file_writer.as_default():
        tf.summary.scalar(label, scalar, step=0)
